@@ -1,105 +1,153 @@
+
 #!/usr/bin/python
 # -*- coding: utf-8; tab-width: 4; indent-tabs-mode: t -*-
 
 """
-The basic interface and implementation for a plugin manager.
+Role
+====
 
-Also define the basic mechanism to add functionalities to the base
-PluginManager. A few *principles* to follow in this case:
+The ``PluginManager`` loads plugins that enforce the `Plugin
+Description Policy`_, and offers the most simple methods to activate
+and deactivate the plugins once they are loaded.
+
+.. note:: It may also classify the plugins in various categories, but
+          this behaviour is optional and if not specified elseway all
+          plugins are stored in the same default category.
+
+.. note:: It is often more useful to have the plugin manager behave
+          like singleton, this functionality is provided by
+          ``PluginManagerSingleton``
+
+
+Plugin Description Policy
+=========================
+
+When creating a ``PluginManager`` instance, one should provide it with
+a list of directories where plugins may be found. In each directory,
+a plugin should contain the following elements:
+
+For a  *Standard* plugin:
+
+  ``myplugin.yapsy-plugin`` 
+ 
+      A *plugin info file* identical to the one previously described.
+ 
+  ``myplugin``
+ 
+      A directory ontaining an actual Python plugin (ie with a
+      ``__init__.py`` file that makes it importable). The upper
+      namespace of the plugin should present a class inheriting the
+      ``IPlugin`` interface (the same remarks apply here as in the
+      previous case).
+
+
+For a *Single file* plugin:
+
+  ``myplugin.yapsy-plugin`` 
+       
+    A *plugin info file* which is identified thanks to its extension,
+    see the `Plugin Info File Format`_ to see what should be in this
+    file.
+    
+  
+    The extension is customisable at the ``PluginManager``'s
+    instanciation, since one may usually prefer the extension to bear
+    the application name.
+  
+  ``myplugin.py``
+  
+     The source of the plugin. This file should at least define a class
+     inheriting the ``IPlugin`` interface. This class will be
+     instanciated at plugin loading and it will be notified the
+     activation/deactivation events.
+
+
+Plugin Info File Format
+-----------------------
+
+The plugin info file gathers, as its name suggests, some basic
+information about the plugin.
+
+- it gives crucial information needed to be able to load the plugin
+
+- it provides some documentation like information like the plugin
+  author's name and a short description fo the plugin functionality.
+
+Here is an example of what such a file should contain::
+
+	  [Core]
+	  Name = My plugin Name
+	  Module = the_name_of_the_pluginto_load_with_no_py_ending
+         
+	  [Documentation]
+	  Description = What my plugin broadly does
+	  Author = My very own name
+	  Version = 0.1
+	  Website = My very own website
+	  Version = the_version_number_of_the_plugin
+	  
+	 
+.. note:: From such plugin descriptions, the ``PluginManager`` will
+          built its own representations of the plugins as instances of
+          the :doc:`PluginInfo` class.
+
+
+Extensibility
+=============
+
+Several mechanisms have been put up to help extending the basic
+functionalities of the proivided classes.
+
+A few *hints* to help you extend those classes:
 
 If the new functionalities do not overlap the ones already
-implemented, then they must be implemented as a Decorator class of the
+implemented, then they should be implemented as a Decorator class of the
 base plugin. This should be done by inheriting the
 ``PluginManagerDecorator``.
 
 If this previous way is not possible, then the functionalities should
 be added as a subclass of ``PluginManager``.
 
-The first method is highly prefered since it makes it possible to have
-a more flexible design where one can pick several functionalities and
-litterally *add* them to get an object corresponding to one's precise
-needs.
+.. note:: The first method is highly prefered since it makes it
+          possible to have a more flexible design where one can pick
+          several functionalities and litterally *add* them to get an
+          object corresponding to one's precise needs.
+
+API
+===
+
 """
 
-import sys, os
+import sys
+import os
 import logging
 import ConfigParser
-import types
 
-from IPlugin import IPlugin
+from yapsy.IPlugin import IPlugin
+from yapsy.PluginInfo import PluginInfo
 
 
-# A forbiden string that can later be used to describe lists of
-# plugins for instance (see ``ConfigurablePluginManager``)
 PLUGIN_NAME_FORBIDEN_STRING=";;"
+"""
+.. warning:: This string (';;' by default) is forbidden in plugin
+             names, and will be usable to describe lists of plugins
+             for instance (see :doc:`ConfigurablePluginManager`)
+"""
 
-class PluginInfo(object):
-	"""
-	Gather some info about a plugin such as its name, author,
-	description...
-	"""
-	
-	def __init__(self, plugin_name, plugin_path):
-		"""
-		Set the namle and path of the plugin as well as the default
-		values for other usefull variables.
-
-		.. warning:: The ``path`` attribute is the full path to the
-		    plugin if it is organised as a directory or the full path
-		    to a file without the ``.py`` extension if the plugin is
-		    defined by a simple file. In the later case, the actual
-		    plugin is reached via ``plugin_info.path+'.py'``.
-			
-		"""
-		self.name = plugin_name
-		self.path = plugin_path
-		self.author		= "Unknown"
-		self.version	= "?.?"
-		self.website	= "None"
-		self.copyright	= "Unknown"
-		self.description = ""
-		self.plugin_object = None
-		self.category     = None
-
-	def _getIsActivated(self):
-		"""
-		Return the activated state of the plugin object.
-		Makes it possible to define a property.
-		"""
-		return self.plugin_object.is_activated
-	is_activated = property(fget=_getIsActivated)
-
-	def setVersion(self, vstring):
-		"""
-		Set the version of the plugin.
-
-		Used by subclasses to provide different handling of the
-		version number.
-		"""
-		self.version = vstring
 
 class PluginManager(object):
 	"""
-	Manage several plugins by ordering them in several categories.
-
+	Manage several plugins by ordering them in categories.
+	
 	The mechanism for searching and loading the plugins is already
 	implemented in this class so that it can be used directly (hence
 	it can be considered as a bit more than a mere interface)
-
-	The file describing a plugin should be written in the sytax
-	compatible with Python's ConfigParser module as in the following
-	example::
-	  
-	  [Core Information]
-	  Name= My plugin Name
-	  Module=the_name_of_the_pluginto_load_with_no_py_ending
-         
-	  [Documentation]
-	  Description=What my plugin broadly does
-	  Author= My very own name
-	  Website= My very own website
-	  Version=the_version_number_of_the_plugin
+	
+	The file describing a plugin must be written in the syntax
+	compatible with Python's ConfigParser module as in the
+	`Plugin Info File Format`_  
 	"""
+	
 
 	def __init__(self, 
 				 categories_filter={"Default":IPlugin}, 
@@ -126,7 +174,7 @@ class PluginManager(object):
 		"""
 		Set the categories of plugins to be looked for as well as the
 		way to recognise them.
-
+		
 		The ``categories_filter`` first defines the various categories
 		in which the plugins will be stored via its keys and it also
 		defines the interface tha has to be inherited by the actual
@@ -138,7 +186,7 @@ class PluginManager(object):
 		# also maps the plugin info files (useful to avoid loading
 		# twice the same plugin...)
 		self._category_file_mapping = {}
-		for categ in categories_filter.keys():
+		for categ in categories_filter:
 			self.category_mapping[categ] = []
 			self._category_file_mapping[categ] = []
 			
@@ -181,14 +229,36 @@ class PluginManager(object):
 		Return the list of all categories.
 		"""
 		return self.category_mapping.keys()
+	
+	def removePluginFromCategory(self,plugin,category_name):
+		"""
+		Remove a plugin from the category where it's assumed to belong.
+		"""
+		self.category_mapping[category_name].remove(plugin)
+		
+		
+	def appendPluginToCategory(self,plugin,category_name):
+		"""
+		Append a new plugin to the given category.
+		"""
+		self.category_mapping[category_name].append(plugin)
 
+	
 	def getPluginsOfCategory(self,category_name):
 		"""
 		Return the list of all plugins belonging to a category.
 		"""
-		return self.category_mapping[category_name]
-
-
+		return self.category_mapping[category_name][:]
+	
+	def getAllPlugins(self):
+		"""
+		Return the list of all plugins (belonging to all categories).
+		"""
+		allPlugins = []
+		for pluginsOfOneCategory in self.category_mapping.itervalues():
+				allPlugins.extend(pluginsOfOneCategory)
+		return allPlugins
+	
 	def _gatherCorePluginInfo(self, directory, filename):
 		"""
 		Gather the core information (name, and module to be loaded)
@@ -260,6 +330,50 @@ class PluginManager(object):
 				plugin_info.description = config_parser.get("Documentation", "Description")
 		return plugin_info
 
+
+
+
+	
+	def getPluginCandidates(self):
+		"""
+		Return the list of possible plugins.
+
+		Each possible plugin (ie a candidate) is described by a 3-uple:
+		(info file path, python file path, plugin info instance)
+
+		.. warning: locatePlugins must be called before !
+		"""
+		if not hasattr(self, '_candidates'):
+			raise ValueError("locatePlugins must be called before getPluginCandidates")
+		return self._candidates[:]
+
+	def removePluginCandidate(self,candidateTuple):
+		"""
+		Remove a given candidate from the list of plugins that should be loaded.
+
+		The candidate must be represented by the same tuple described
+		in ``getPluginCandidates``.
+		
+		.. warning: locatePlugins must be called before !
+		"""
+		if not hasattr(self, '_candidates'):
+			raise ValueError("locatePlugins must be called before removePluginCandidate")
+		self._candidates.remove(candidateTuple)
+
+	def appendPluginCandidate(self,candidateTuple):
+		"""
+		Append a new candidate to the list of plugins that should be loaded.
+		
+		The candidate must be represented by the same tuple described
+		in ``getPluginCandidates``.
+		
+		.. warning: locatePlugins must be called before !
+		"""
+		if not hasattr(self, '_candidates'):
+			raise ValueError("locatePlugins must be called before removePluginCandidate")
+		self._candidates.append(candidateTuple)
+		
+		
 	def locatePlugins(self):
 		"""
 		Walk through the plugins' places and look for plugins.
@@ -331,7 +445,8 @@ class PluginManager(object):
 			if "__init__" in  os.path.basename(candidate_filepath):
 				sys.path.append(plugin_info.path)				
 			try:
-				execfile(candidate_filepath+".py",candidate_globals)
+				candidateMainFile = open(candidate_filepath+".py","r")	
+				exec(candidateMainFile,candidate_globals)
 			except Exception,e:
 				logging.debug("Unable to execute the code in plugin: %s" % candidate_filepath)
 				logging.debug("\t The following problem occured: %s %s " % (os.linesep, e))
@@ -342,9 +457,9 @@ class PluginManager(object):
 			if "__init__" in  os.path.basename(candidate_filepath):
 				sys.path.remove(plugin_info.path)
 			# now try to find and initialise the first subclass of the correct plugin interface
-			for element in candidate_globals.values():
+			for element in candidate_globals.itervalues():
 				current_category = None
-				for category_name in self.categories_interfaces.keys():
+				for category_name in self.categories_interfaces:
 					try:
 						is_correct_subclass = issubclass(element, self.categories_interfaces[category_name])
 					except:
@@ -382,7 +497,7 @@ class PluginManager(object):
 		"""
 		Get the plugin correspoding to a given category and name
 		"""
-		if self.category_mapping.has_key(category):
+		if category in self.category_mapping:
 			for item in self.category_mapping[category]:
 				if item.name == name:
 					return item
@@ -406,7 +521,7 @@ class PluginManager(object):
 		"""
 		Desactivate a plugin corresponding to a given category + name.
 		"""
-		if self.category_mapping.has_key(category):
+		if category in self.category_mapping:
 			plugin_to_deactivate = None
 			for item in self.category_mapping[category]:
 				if item.name == name:
@@ -419,76 +534,6 @@ class PluginManager(object):
 		return None
 
 
-class PluginManagerDecorator(object):
-	"""
-	Make it possible to add several responsibilities to a plugin
-	manager object in a more flexible way than by mere
-	subclassing. This is indeed an implementation of the Decorator
-	Design Patterns.
-
-	
-	There is also an additional mechanism that allows for the
-	automatic creation of the object to be decorated when this object
-	is an instance of PluginManager (and not an instance of its
-	subclasses). This way we can keep the plugin managers creation
-	simple when the user don't want to mix a lot of 'enhancements' on
-	the base class.
-	"""
-
-	def __init__(self,decorated_object=None,
-				 # The following args will only be used if we need to
-				 # create a default PluginManager
-				 categories_filter={"Default":IPlugin}, 
-				 directories_list=[os.path.dirname(__file__)], 
-				 plugin_info_ext="yapsy-plugin"):
-		"""
-		Mimics the PluginManager's __init__ method and wraps an
-		instance of this class into this decorator class.
-		
-		  - *If the decorated_object is not specified*, then we use the
-		    PluginManager class to create the 'base' manager, and to do
-		    so we will use the arguments: ``categories_filter``,
-		    ``directories_list``, and ``plugin_info_ext`` or their
-		    default value if they are not given.
-
-		  - *If the decorated object is given*, these last arguments are
-		    simply **ignored** !
-
-		All classes (and especially subclasses of this one) that want
-		to be a decorator must accept the decorated manager as an
-		object passed to the init function under the exact keyword
-		``decorated_object``.
-		"""
-		
-		if decorated_object is None:
-			logging.debug("Creating a default PluginManager instance to be decorated.")
-			decorated_object = PluginManager(categories_filter, 
-											 directories_list,
-											 plugin_info_ext)
-		self._component = decorated_object
-
-	def __getattr__(self,name):
-		"""
-		Decorator trick copied from:
-		http://www.pasteur.fr/formation/infobio/python/ch18s06.html
-		"""
-# 		print "looking for %s in %s" % (name, self.__class__)
-		return getattr(self._component,name)
-		
-		
-	def collectPlugins(self):
-		"""
-		This function will usually be a shortcut to successively call
-		``self.locatePlugins`` and then ``self.loadPlugins`` which are
-		very likely to be redefined in each new decorator.
-
-		So in order for this to keep on being a "shortcut" and not a
-		real pain, I'm redefining it here.
-		"""
-		self.locatePlugins()
-		self.loadPlugins()
-
-
 class PluginManagerSingleton(object):
 	"""
 	Singleton version of the most basic plugin manager.
@@ -574,3 +619,9 @@ class PluginManagerSingleton(object):
 			logging.debug("PluginManagerSingleton initialised")
 		return self.__instance
 	get = classmethod(get)
+
+
+# For backward compatility import the most basic decorator (it changed
+# place as of v1.8)
+from yapsy.PluginManagerDecorator import PluginManagerDecorator
+
